package org.seasar.quartz;

import junit.framework.TestCase;
import org.quartz.Scheduler;
import org.quartz.SchedulerFactory;
import org.quartz.impl.StdSchedulerFactory;
import org.seasar.framework.container.ComponentDef;
import org.seasar.framework.container.MetaDef;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.impl.ArgDefImpl;
import org.seasar.framework.container.impl.ComponentDefImpl;
import org.seasar.framework.container.impl.MetaDefImpl;
import org.seasar.framework.container.impl.S2ContainerImpl;

public class QuartzConfigurerTest extends TestCase {

    public void testInit() throws Exception {
	SchedulerFactory schedulerFactory = new StdSchedulerFactory();
	Scheduler scheduler = schedulerFactory.getScheduler();

	S2Container childContainer = new S2ContainerImpl();
	childContainer.register(Object.class);
	ComponentDef componentDef = new ComponentDefImpl(String.class);
	componentDef.addArgDef(new ArgDefImpl("childContainer"));
	MetaDef metaDef = new MetaDefImpl("quartz:job");
	metaDef.addMetaDef(new MetaDefImpl("methodName", "toString"));
	metaDef.addMetaDef(new MetaDefImpl("cronExpression", "0/10 * * ? * *"));
	componentDef.addMetaDef(metaDef);
	metaDef = new MetaDefImpl("quartz:job");
	metaDef.addMetaDef(new MetaDefImpl("methodName", "getClass"));
	metaDef.addMetaDef(new MetaDefImpl("cronExpression", "0/15 * * ? * *"));
	componentDef.addMetaDef(metaDef);
	childContainer.register(componentDef);
	S2Container parentContainer = new S2ContainerImpl();
	parentContainer.register(Object.class);
	componentDef = new ComponentDefImpl(String.class);
	componentDef.addArgDef(new ArgDefImpl("parentContainer"));
	metaDef = new MetaDefImpl("quartz:job");
	metaDef.addMetaDef(new MetaDefImpl("methodName", "hashCode"));
	metaDef.addMetaDef(new MetaDefImpl("cronExpression", "0/20 * * ? * *"));
	componentDef.addMetaDef(metaDef);
	parentContainer.register(componentDef);
	parentContainer.include(childContainer);

	QuartzConfigurer quartzConfigurer = new QuartzConfigurer();
	quartzConfigurer.init(scheduler, childContainer);

	assertEquals(3, scheduler.getJobNames("DEFAULT").length);
	assertEquals(3, scheduler.getTriggerNames("DEFAULT").length);
	assertEquals("parentContainer", scheduler.getJobDetail("quartz:job:job-detail:1", "DEFAULT").getJobDataMap().getString("org.seasar.quartz.component"));
	assertEquals("childContainer", scheduler.getJobDetail("quartz:job:job-detail:2", "DEFAULT").getJobDataMap().getString("org.seasar.quartz.component"));
	assertEquals("childContainer", scheduler.getJobDetail("quartz:job:job-detail:3", "DEFAULT").getJobDataMap().getString("org.seasar.quartz.component"));
	assertEquals("hashCode", scheduler.getJobDetail("quartz:job:job-detail:1", "DEFAULT").getJobDataMap().getString("org.seasar.quartz.methodName"));
	assertEquals("toString", scheduler.getJobDetail("quartz:job:job-detail:2", "DEFAULT").getJobDataMap().getString("org.seasar.quartz.methodName"));
	assertEquals("getClass", scheduler.getJobDetail("quartz:job:job-detail:3", "DEFAULT").getJobDataMap().getString("org.seasar.quartz.methodName"));
	assertEquals(1, scheduler.getTriggersOfJob("quartz:job:job-detail:1", "DEFAULT").length);
	assertEquals(1, scheduler.getTriggersOfJob("quartz:job:job-detail:2", "DEFAULT").length);
	assertEquals(1, scheduler.getTriggersOfJob("quartz:job:job-detail:3", "DEFAULT").length);
	assertEquals("quartz:job:trigger:1", scheduler.getTriggersOfJob("quartz:job:job-detail:1", "DEFAULT")[0].getName());
	assertEquals("quartz:job:trigger:2", scheduler.getTriggersOfJob("quartz:job:job-detail:2", "DEFAULT")[0].getName());
	assertEquals("quartz:job:trigger:3", scheduler.getTriggersOfJob("quartz:job:job-detail:3", "DEFAULT")[0].getName());
	assertEquals("0/20 * * ? * *", ((CronTrigger) scheduler.getTrigger("quartz:job:trigger:1", "DEFAULT")).getCronExpression());
	assertEquals("0/10 * * ? * *", ((CronTrigger) scheduler.getTrigger("quartz:job:trigger:2", "DEFAULT")).getCronExpression());
	assertEquals("0/15 * * ? * *", ((CronTrigger) scheduler.getTrigger("quartz:job:trigger:3", "DEFAULT")).getCronExpression());
    }
}
